=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-05-08 01:57:31 +0000
+++ bzrlib/bzrdir.py	2006-05-17 04:09:58 +0000
@@ -118,7 +118,11 @@
         if local_repo:
             # may need to copy content in
             if force_new_repo:
-                local_repo.clone(result, revision_id=revision_id, basis=basis_repo)
+                result_repo = local_repo.clone(
+                    result,
+                    revision_id=revision_id,
+                    basis=basis_repo)
+                result_repo.set_make_working_trees(local_repo.make_working_trees())
             else:
                 try:
                     result_repo = result.find_repository()
@@ -130,7 +134,11 @@
                     result_repo.fetch(local_repo, revision_id=revision_id)
                 except errors.NoRepositoryPresent:
                     # needed to make one anyway.
-                    local_repo.clone(result, revision_id=revision_id, basis=basis_repo)
+                    result_repo = local_repo.clone(
+                        result,
+                        revision_id=revision_id,
+                        basis=basis_repo)
+                    result_repo.set_make_working_trees(local_repo.make_working_trees())
         # 1 if there is a branch present
         #   make sure its content is available in the target repository
         #   clone it.
@@ -567,11 +575,12 @@
             # no repo available, make a new one
             result.create_repository()
         elif source_repository is not None and result_repo is None:
-            # have soure, and want to make a new target repo
-            source_repository.clone(result,
-                                    revision_id=revision_id,
-                                    basis=basis_repo)
-        else:
+            # have source, and want to make a new target repo
+            # we dont clone the repo because that preserves attributes
+            # like is_shared(), and we have not yet implemented a 
+            # repository sprout().
+            result_repo = result.create_repository()
+        if result_repo is not None:
             # fetch needed content into target.
             if basis_repo:
                 # XXX FIXME RBC 20060214 need tests for this when the basis

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-05-05 04:00:11 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-05-17 04:09:58 +0000
@@ -171,6 +171,32 @@
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertNotEqual(dir.transport.base, shared_repo.bzrdir.transport.base)
         self.assertTrue(shared_repo.has_revision('1'))
+
+    def test_clone_bzrdir_repository_branch_only_source_under_shared(self):
+        try:
+            shared_repo = self.make_repository('shared', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        tree.bzrdir.open_branch().set_revision_history([])
+        tree.set_last_revision(None)
+        tree.commit('revision 2', rev_id='2')
+        tree.bzrdir.open_repository().copy_content_into(shared_repo)
+        shared_repo.set_make_working_trees(False)
+        self.assertFalse(shared_repo.make_working_trees())
+        self.assertTrue(shared_repo.has_revision('1'))
+        dir = self.make_bzrdir('shared/source')
+        dir.create_branch()
+        target = dir.clone(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertNotEqual(dir.transport.base, shared_repo.bzrdir.transport.base)
+        branch = target.open_branch()
+        self.assertTrue(branch.repository.has_revision('1'))
+        self.assertFalse(branch.repository.make_working_trees())
+        self.assertTrue(branch.repository.is_shared())
         
     def test_clone_bzrdir_repository_under_shared_force_new_repo(self):
         tree = self.make_branch_and_tree('commit_tree')
@@ -480,6 +506,32 @@
         self.assertNotEqual(dir.transport.base, shared_repo.bzrdir.transport.base)
         self.assertTrue(shared_repo.has_revision('1'))
 
+    def test_sprout_bzrdir_repository_branch_only_source_under_shared(self):
+        try:
+            shared_repo = self.make_repository('shared', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        tree.bzrdir.open_branch().set_revision_history([])
+        tree.set_last_revision(None)
+        tree.commit('revision 2', rev_id='2')
+        tree.bzrdir.open_repository().copy_content_into(shared_repo)
+        shared_repo.set_make_working_trees(False)
+        self.assertFalse(shared_repo.make_working_trees())
+        self.assertTrue(shared_repo.has_revision('1'))
+        dir = self.make_bzrdir('shared/source')
+        dir.create_branch()
+        target = dir.sprout(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertNotEqual(dir.transport.base, shared_repo.bzrdir.transport.base)
+        branch = target.open_branch()
+        self.assertTrue(branch.repository.has_revision('1'))
+        self.assertTrue(branch.repository.make_working_trees())
+        self.assertFalse(branch.repository.is_shared())
+
     def test_sprout_bzrdir_repository_under_shared_force_new_repo(self):
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))

